#
# ff-survive.R, 20 Jul 13
#
# R code for book "Empirical Software Engineering using R"
# Derek M. Jones, http://shape-of-code.coding-guidelines.com
#
# Uses data from:
# After-Life Vulnerabilities: A Study on Firefox Evolution, its Vulnerabilities, and Fixes
# by Fabio Massacci and Stephen Neuhaud and Viet Hung Nguyen
#

root="/usr1/rbook/examples/faults/milk-wine/"

par(mfcol=c(1,1))
par(col="black")
par(bty="l")
par(las=1)
par(pch=3)

study_start=as.Date("9-November-2004", format="%d-%B-%Y")
study_end=as.Date("1-October-2010", format="%d-%B-%Y")

# version,v1.0,v1.5,v2.0,v3.0,v3.5,v3.6
# Treat column 1 as the rownames
version_fault=read.csv(paste0(root, "ff-version-fault.csv"), row.names=1, as.is=TRUE)
vf_names=colnames(version_fault)

# version,release,retire
# date format: 9-November-2004
# Treat column 1 as the rownames
version_dates=read.csv(paste0(root, "ff-release-retire.csv"), row.names=1, as.is=TRUE)
version_dates$release=as.Date(version_dates$release, format="%d-%B-%Y")
version_dates$retire=as.Date(version_dates$retire, format="%d-%B-%Y")
version_dates$retire[version_dates$retire > study_end]=study_end

# Return dataframe of fault_count rows, each row containing a fault event.
# Gets passed a single cell having an array type
mk_cell_survive=function(fault_count)
{
if (is.na(fault_count))
   return(NULL)

# Use names of row/column to index into version/date array
src_cluster=rownames(fault_count)
rep_cluster=colnames(fault_count)
src_start_date=version_dates[src_cluster, 1]
rep_start_date=version_dates[rep_cluster, 1]
rep_end_date=version_dates[rep_cluster, 2]

# convert from array
fault_count=as.integer(fault_count)

# print(fault_count)

# A survivor?
if (fault_count == 0)
   {
   survive_list=data.frame(rep_date=rep_end_date)
   survive_list$start=as.numeric(src_start_date-study_start)
   survive_list$end=as.numeric(survive_list$rep_date-study_start)
   survive_list$src_id=src_cluster
   survive_list$rep_id=rep_cluster
   survive_list$event=0

   return(survive_list)
   }

# Calculate  time interval information
start_offset=as.numeric(rep_start_date)
end_offset=as.numeric(rep_end_date)
v_interval=end_offset-start_offset
f_width=v_interval / fault_count

# print(c(start_offset, end_offset, v_interval, f_width))

# end offset is from start of report period
# Assume uniform time spacing between faults
survive_list=data.frame(rep_date=rep_start_date+(1:fault_count)*f_width)
survive_list$end=as.numeric(survive_list$rep_date-study_start)
# start offset is start of version where source was created
survive_list$start=as.numeric(src_start_date-study_start)
survive_list$src_id=src_cluster
survive_list$rep_id=rep_cluster
survive_list$event=1
survive_list$LOC=NA

return(survive_list)
}


# Take a row and iterate over each cell (column)
mk_row_survive=function(version_row)
{
t=adply(version_row, 2, mk_cell_survive)

return(t)
}


# Take an array and iterate over each row
mk_survive=function(version_array)
{
t=adply(version_array, 1, mk_row_survive)

return(t)
}

ff_faults=mk_survive(version_fault)

t=ff_faults[order(ff_faults$rep_date), ]

f_sur=Surv(ff_faults$start, ff_faults$end, ff_faults$event)

#f_fit=survfit(f_sur ~ 1)
f_fit=survfit(f_sur ~ 1+cluster(src_id), data=ff_faults)
plot(f_fit)


# Take the original data format and convert to the one used here.
# Convert a list of values into a 2-D matrix
#
# src_version=read.csv(paste0(root, "orig-ff-src-version.csv"), as.is=TRUE)
# 
# q=matrix(data=0, nrow=6, ncol=6)
# rownames(q)=c("v1.0", "v1.5", "v2.0", "v3.0", "v3.5", "v3.6")
# colnames(q)=c("v1.0", "v1.5", "v2.0", "v3.0", "v3.5", "v3.6")
# 
# q[cbind(src_version[, 2], src_version[, 1])] = src_version[, 3]
# write.csv(q, file="ff-src-version.csv")

src_version=read.csv(paste0(root, "ff-src-version.csv"), row.names=1, as.is=TRUE)

ff_faults$LOC=src_version[cbind(ff_faults$src_id, ff_faults$rep_id)]


# Return market share of highest version in use when fault reported
get_market_share=function(row)
{
t=subset(browser_ms, browser_ms$browser == row$rep_id)
# Find the market_shared that occured closest to row$end
res=t$market_share[which.min(abs(t$date-row$end))]
return(res)
}


browser_ms=read.csv(paste0(root, "w3stats_browser.csv"), as.is=TRUE)
browser_ms$date=as.Date(browser_ms$date, format="%m/%d/%Y")-study_start
# plot(browser_ms$market_share[browser_ms$browser == "v1.0"])

# had problems using aaply :-(
t=sapply(1:nrow(ff_faults), function(X) get_market_share(ff_faults[X, ]))
ff_faults$market_share=t


# t=ddply(browser_ms, .(date), function(df) sum(df$market_share))
# head(t, n=40)

i_users=read.csv(paste0(root, "itu-internet-users.csv"), as.is=TRUE)
i_users$year=as.Date(i_users$year)
user_mod=lm(Developed ~ year, data=i_users)

ff_faults$i_users=predict(user_mod, newdata=data.frame(year=ff_faults$rep_date))


t=subset(ff_faults, src_id == "v1.0")
t=subset(q, src_id == "v1.0")
t$round_date=as.numeric(t$rep_date) %/% 2
w=daply(t, .(round_date), function(df) sum(df$market_share))

plot(t$LOC*t$i_users*t$market_share)

# t=subset(ff_faults, rep_id == "v1.0")
# plot(t$market_share)


ff_cox=coxph(f_sur ~ LOC+cluster(src_id), data=ff_faults)

